/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.applicationprocessor.processor;

import java.util.List;

import org.tinygroup.appconfig.AppConfig;
import org.tinygroup.appconfig.util.AppConfigUtil;
import org.tinygroup.application.Application;
import org.tinygroup.application.ApplicationProcessor;
import org.tinygroup.bu.BusinessUnitManagerFactory;
import org.tinygroup.bu.util.BusinessUnitUtil;
import org.tinygroup.fileresolver.FileProcessor;
import org.tinygroup.logger.LogLevel;
import org.tinygroup.logger.Logger;
import org.tinygroup.logger.LoggerFactory;
import org.tinygroup.parser.filter.NameFilter;
import org.tinygroup.xmlparser.node.XmlNode;

public class BuProcessor implements ApplicationProcessor {
	private static final String BU_MANAGEMENT_NODE_NAME = "bu-management";
	private static Logger logger = LoggerFactory.getLogger(BuProcessor.class);
	private XmlNode buNode;
	private Application application;

	public String getNodeName() {
		return BU_MANAGEMENT_NODE_NAME;
	}

	public void setConfiguration(XmlNode xmlNode) {
		this.buNode = xmlNode;
	}

	public XmlNode getConfiguration() {
		return buNode;
	}

	public void start() {
		logger.logMessage(LogLevel.INFO, "启动BuProcessor");
		// 由于bu均从lib下扫描了，所以这段逻辑可以注释先
		// loadConfigResolverProcessors();
		// BusinessUnitManagerFactory.getManager().load(getPath());
		AppConfigUtil.config(this, BusinessUnitManagerFactory.getManager());
		BusinessUnitManagerFactory.getManager().install();
		logger.logMessage(LogLevel.INFO, "启动BuProcessor完毕");
	}

	public void stop() {

	}

	public String getPath() {
		if (buNode != null) {
			String path = buNode.getAttribute("bu-path");
			if (path != null && path.length() > 0) {
				return path;
			}
		}
		return BusinessUnitUtil.getBuDir();

	}

	private void loadConfigResolverProcessors() {
		if (buNode == null) {
			return;
		}
		NameFilter<XmlNode> nameFilter = new NameFilter<XmlNode>(buNode);
		XmlNode fileProcessors = nameFilter.findNode("file-processors");
		loadResolverProcessor(fileProcessors, BusinessUnitManagerFactory
				.getManager().getResourceProcessors());
		XmlNode fileRemoveProcessors = nameFilter
				.findNode("file-remove-processor");
		loadResolverProcessor(fileRemoveProcessors, BusinessUnitManagerFactory
				.getManager().getResourceRemoveProcessors());
	}

	private void loadResolverProcessor(XmlNode fileProcessorsNode,
			List<FileProcessor> processors) {
		if (fileProcessorsNode == null) {
			return;
		}
		NameFilter<XmlNode> nameFilter = new NameFilter<XmlNode>(
				fileProcessorsNode);
		List<XmlNode> fileProcessorNodes = nameFilter
				.findNodeList("file-processor");
		if (fileProcessorNodes == null) {
			return;
		}
		for (XmlNode fileResolver : fileProcessorNodes) {
			String enable = fileResolver.getAttribute("enable");
			if (enable != null && enable.equalsIgnoreCase("false")) {
				continue;
			} else {
				String className = fileResolver.getAttribute("class-name");
				if (className != null && className.length() > 0) {
					logger.logMessage(LogLevel.INFO, "正在加载插件文件处理器[{0}]...",
							className);
					loadFileProcessor(className, fileResolver, processors);
					logger.logMessage(LogLevel.INFO, "文件处理器[{0}]加载完毕",
							className);
				}
			}
		}
	}

	protected ClassLoader getThisClassLoader() {
		return this.getClass().getClassLoader();
	}

	private void loadFileProcessor(String className, XmlNode fileResolverNode,
			List<FileProcessor> processors) {
		try {
			FileProcessor fileProcessor = (FileProcessor) getThisClassLoader()
					.loadClass(className).newInstance();
			if (fileProcessor instanceof AppConfig) {
				AppConfig appconfig = (AppConfig) fileProcessor;
				if (buNode != null) {
					NameFilter<XmlNode> nameFilter = new NameFilter<XmlNode>(
							buNode);
					XmlNode fileConfig = nameFilter.findNode(appconfig
							.getNodeName());
					appconfig.setConfiguration(fileConfig);
				}

			}
			processors.add(fileProcessor);
		} catch (Exception e) {
			logger.errorMessage("加载[{0}]失败！", e, className);
		}
	}

	public void setApplication(Application application) {
		this.application = application;

	}
}
