/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.applicationprocessor.processor;

import java.io.File;
import java.util.List;

import org.tinygroup.appconfig.AppConfig;
import org.tinygroup.appconfig.util.AppConfigUtil;
import org.tinygroup.application.Application;
import org.tinygroup.application.ApplicationProcessor;
import org.tinygroup.fileresolver.FileProcessor;
import org.tinygroup.logger.LogLevel;
import org.tinygroup.logger.Logger;
import org.tinygroup.logger.LoggerFactory;
import org.tinygroup.parser.filter.NameFilter;
import org.tinygroup.plugin.PluginManager;
import org.tinygroup.xmlparser.node.XmlNode;

public class PluginMgtProcessor implements ApplicationProcessor {
	private static final String PLUGIN_MANAGEMENT_NODE_NAME = "plugin-management";
	private static Logger logger = LoggerFactory
			.getLogger(PluginMgtProcessor.class);
	private XmlNode pluginMgtNode;
	private PluginManager pluginManager;
	private Application application;

	public void setApplication(Application application) {
		this.application = application;

	}

	public void stop() {
		if (pluginManager != null) {
			pluginManager.stop();
		}
	}

	public void setPluginManager(PluginManager pluginManager) {
		this.pluginManager = pluginManager;
	}

	public void start() {

		logger.logMessage(LogLevel.INFO, "启动插件管理器");

		pluginManager.load(null);
		AppConfigUtil.config(this, pluginManager);
		pluginManager.start();
		logger.logMessage(LogLevel.INFO, "启动插件管理器成功");
	}

	private void loadPluginResolverProcessors(String processorRootNodeName,
			List<FileProcessor> processors) {

		NameFilter<XmlNode> nameFilter = new NameFilter<XmlNode>(pluginMgtNode);
		XmlNode fileProcessorsNode = nameFilter.findNode(processorRootNodeName);
		if (fileProcessorsNode == null){
			return;
		}
		NameFilter<XmlNode> nodeNameFileter = new NameFilter<XmlNode>(
				fileProcessorsNode);
		List<XmlNode> fileProcessors = nodeNameFileter
				.findNodeList("file-processor");
		if (fileProcessors == null){
			return;
		}
		for (XmlNode fileResolver : fileProcessors) {
			String enable = fileResolver.getAttribute("enable");
			if (enable != null && enable.equalsIgnoreCase("false")) {
				continue;
			} else {
				String className = fileResolver.getAttribute("class-name");
				if (className != null && className.length() > 0) {
					logger.logMessage(LogLevel.INFO, "正在加载插件文件处理器[{0}]...",
							className);
					loadFileResolver(className, fileResolver, processors);
					logger.logMessage(LogLevel.INFO, "文件处理器[{0}]加载完毕",
							className);
				}
			}
		}
	}

	private void loadFileResolver(String className, XmlNode fileResolverNode,
			List<FileProcessor> processors) {
		try {
			FileProcessor fileResolver = (FileProcessor) Class.forName(
					className).newInstance();
			if (fileResolver instanceof AppConfig) {
				AppConfigUtil.config(this, (AppConfig) fileResolver);
			}
			processors.add(fileResolver);
		} catch (Exception e) {
			logger.errorMessage("加载[{0}]失败！", e, className);
		}
	}

	public void setConfiguration(XmlNode xmlNode) {
		this.pluginMgtNode = xmlNode;
	}

	private String getWebRoot() {

		String webRoot = application.getContext().get("TINY_WEBROOT");
		String path = "";
		if (webRoot != null) {
			path = webRoot + File.separator + "WEB-INF";
		} else {
			path = PluginMgtProcessor.class.getClassLoader().getResource("")
					.getPath();
		}
		return path;
	}

	public String getNodeName() {
		return PLUGIN_MANAGEMENT_NODE_NAME;
	}

	public XmlNode getConfiguration() {
		return pluginMgtNode;
	}

}
